local ADDON, Package = ...

local Kernel = Package.Kernel
local Module = {}

function Module.configure_icon_drawer()
  local f = function (self, ...)

    local children = {Minimap:GetChildren()}

    local defaults = {
      MiniMapMailFrame = true,
      MiniMapBattlefieldFrame = true,
      MinimapBackdrop = true,
      TimeManagerClockButton = true
    }

    local button = CreateFrame("CheckButton", ADDON .. "MinimapIcon", MinimapBackdrop)
    button:SetSize(32, 32)
    button:SetPoint("CENTER", -60, -13)
    button:SetNormalTexture("Interface\\Minimap\\MiniMap-TrackingBorder")
    button:SetHighlightTexture("Interface\\Minimap\\UI-Minimap-ZoomButton-Highlight")
    button:SetScript("OnClick", function (self)
      for _, frame in pairs(self.icons) do
        frame:SetShown(self:GetChecked())
      end
    end)
    button.icons = {}

    local length = button:CreateFontString(nil, "ARTWORK", "GameFontNormalTiny")
    length:SetPoint("CENTER")

    local normal_texture = button:GetNormalTexture()
    normal_texture:ClearAllPoints()
    normal_texture:SetPoint("CENTER", 11, -11)
    normal_texture:SetSize(52, 52)

    local background = button:CreateTexture(nil, "BACKGROUND")
    background:SetTexture("Interface\\Minimap\\UI-Minimap-Background")
    background:SetPoint("TOPLEFT", 4, -4)
    background:SetPoint("BOTTOMRIGHT", -4, 4)

    local relative = button

    for _, frame in pairs(children) do
      if not defaults[frame:GetName()] and frame:GetName():match("LibDBIcon") then
        frame:Hide()
        frame:ClearAllPoints()
        frame:SetPoint("RIGHT", relative, "LEFT", 0, 0)

        frame.Show = Kernel.noop
        frame.Hide = Kernel.noop
        frame.SetPoint = Kernel.noop

        table.insert(button.icons, frame)

        relative = frame
      end
    end

    length:SetText(#button.icons)
  end

  local agent = CreateFrame("Frame")
  agent:RegisterEvent("PLAYER_ENTERING_WORLD")
  agent:SetScript("OnEvent", function (self, ...)
    f(self, ...)

    agent:UnregisterEvent("PLAYER_ENTERING_WORLD")
    agent:SetScript("OnEvent", nil)
  end)
end

Package.Minimap = Module
